macro_rules! read_varint {
    ($type:ty, $self:expr) => {{
        let mut shift: $type = 0;
        let mut decoded: $type = 0;
        let mut size: usize = 0;

        loop {
            let next = DataReader::read_byte($self)?;
            size += 1;

            if shift >= (std::mem::size_of::<$type>() * 8) as $type {
                return Err(ProtocolError::VarIntError);
            }

            decoded |= ((next & 0x7F) as $type) << shift;

            if next & 0x80 == 0x80 {
                shift += 7;
            } else {
                return Ok((decoded, size));
            }
        }
    }};
}

macro_rules! write_varint {
    ($type:ty, $self:expr, $value:expr) => {{
        let mut value: $type = $value;

        if value == 0 {
            DataWriter::write_byte($self, 0)
        } else {
            while value >= 0x80 {
                DataWriter::write_byte($self, ((value & 0x7F) as u8) | 0x80)?;
                value >>= 7;
            }

            DataWriter::write_byte($self, (value & 0x7F) as u8)
        }
    }};
}

pub(crate) use read_varint;
pub(crate) use write_varint;
